@@ -120,6 +120,7 @@ r"""输出内容到用户浏览器
 .. autofunction::  output
 
 """
+import html
 import io
 import logging
 import string
@@ -143,7 +144,7 @@ __all__ = ['Position', 'remove', 'scroll_to',
            'put_text', 'put_html', 'put_code', 'put_markdown', 'use_scope', 'set_scope', 'clear', 'remove',
            'put_table', 'put_buttons', 'put_image', 'put_file', 'PopupSize', 'popup',
            'close_popup', 'put_widget', 'put_collapse', 'put_link', 'put_scrollable', 'style', 'put_column',
-           'put_row', 'put_grid', 'column', 'row', 'grid', 'span', 'put_processbar', 'set_processbar', 'put_loading',
+           'put_row', 'put_grid', 'span', 'put_processbar', 'set_processbar', 'put_loading',
            'output', 'toast', 'get_scope']
 
 
@@ -196,9 +197,10 @@ def set_scope(name, container_scope=Scope.Current, position=OutputPosition.BOTTO
     """创建一个新的scope.
 
     :param str name: scope名
-    :param int/str container_scope: 此scope的父scope. 可以直接指定父scope名或使用 `Scope` 常量. scope不存在时，不进行任何操作.
+    :param int/str container_scope: 指定此scope的父scope. 可以直接指定父scope名或使用int索引运行时scope栈(参见 :ref:`输出函数的scope相关参数 <scope_param>`). scope不存在时，不进行任何操作.
     :param int position: 在父scope中创建此scope的位置.
-       `OutputPosition.TOP` : 在父scope的顶部创建, `OutputPosition.BOTTOM` : 在父scope的尾部创建
+       可选值: `OutputPosition.TOP` : 在父scope的顶部创建, `OutputPosition.BOTTOM`: 在父scope的底部创建。
+       也可以直接使用int来索引位置(参见 :ref:`输出函数的scope相关参数 <scope_param>`)
     :param str if_exist: 已经存在 ``name`` scope 时如何操作:
 
         - `None` 表示不进行任何操作
@@ -206,6 +208,7 @@ def set_scope(name, container_scope=Scope.Current, position=OutputPosition.BOTTO
         - `'clear'` 表示将旧scope的内容清除，不创建新scope
 
        默认为 `None`
+
     """
     if isinstance(container_scope, int):
         container_scope = get_current_session().get_scope_name(container_scope)
@@ -233,7 +236,7 @@ def get_scope(stack_idx=Scope.Current):
 def clear(scope=Scope.Current):
     """清空scope内容
 
-    :param int/str scope: 可以直接指定scope名或使用 `Scope` 常量
+    :param int/str scope: 可以直接指定scope名或使用int索引运行时scope栈(参见 :ref:`输出函数的scope相关参数 <scope_param>`)
     """
     if isinstance(scope, int):
         scope = get_current_session().get_scope_name(scope)
@@ -241,7 +244,10 @@ def clear(scope=Scope.Current):
 
 
 def remove(scope=Scope.Current):
-    """移除Scope"""
+    """移除Scope
+
+    :param int/str scope: 可以直接指定scope名或使用int索引运行时scope栈(参见 :ref:`输出函数的scope相关参数 <scope_param>`)
+    """
     if isinstance(scope, int):
         scope = get_current_session().get_scope_name(scope)
     assert scope != 'ROOT', "Can not remove `ROOT` scope."
@@ -249,16 +255,15 @@ def remove(scope=Scope.Current):
 
 
 def scroll_to(scope=Scope.Current, position=Position.TOP):
-    """scroll_to(scope, position=Position.TOP)
-
+    """
     将页面滚动到 ``scope`` Scope处
 
-    :param str/int scope: Scope名
+    :param str/int scope: 可以直接指定scope名或使用int索引运行时scope栈(参见 :ref:`输出函数的scope相关参数 <scope_param>`)
     :param str position: 将Scope置于屏幕可视区域的位置。可用值：
 
-       * ``Position.TOP`` : 滚动页面，让Scope位于屏幕可视区域顶部
-       * ``Position.MIDDLE`` : 滚动页面，让Scope位于屏幕可视区域中间
-       * ``Position.BOTTOM`` : 滚动页面，让Scope位于屏幕可视区域底部
+       * ``'top'`` : 滚动页面，让Scope位于屏幕可视区域顶部
+       * ``'middle'`` : 滚动页面，让Scope位于屏幕可视区域中间
+       * ``'bottom'`` : 滚动页面，让Scope位于屏幕可视区域底部
     """
     if isinstance(scope, int):
         scope = get_current_session().get_scope_name(scope)
@@ -298,7 +303,7 @@ def put_text(*texts, sep=' ', inline=False, scope=Scope.Current, position=Output
 
     :param texts: 要输出的内容。类型可以为任意对象，对非字符串对象会应用 `str()` 函数作为输出值。
     :param str sep: 输出分隔符
-    :param bool inline: 文本行末不换行。默认换行
+    :param bool inline: 将文本作为行内元素(连续输出的文本显示在相同的段落中)。默认每次输出的文本都作为一个独立的段落
     :param int/str scope: 内容输出的目标scope，若scope不存在，则不进行任何输出操作。
 
        可以直接指定目标Scope名，或者使用int通过索引Scope栈来确定Scope：0表示最顶层也就是ROOT Scope，-1表示当前Scope，-2表示进入当前Scope的前一个Scope，...
@@ -313,42 +318,59 @@ def put_text(*texts, sep=' ', inline=False, scope=Scope.Current, position=Output
     return Output(spec)
 
 
-def put_html(html, scope=Scope.Current, position=OutputPosition.BOTTOM) -> Output:
+def put_html(html, sanitize=False, scope=Scope.Current, position=OutputPosition.BOTTOM) -> Output:
     """
     输出Html内容。
 
     与支持通过Html输出内容到 `Jupyter Notebook <https://nbviewer.jupyter.org/github/ipython/ipython/blob/master/examples/IPython%20Kernel/Rich%20Output.ipynb#HTML>`_ 的库兼容。
 
-    :param html: html字符串或实现了 `IPython.display.HTML` 接口的类的实例
+    :param html: html字符串或实现了 `IPython.display.HTML` 接口的实例
+    :param bool sanitize: 是否使用 `DOMPurify <https://github.com/cure53/DOMPurify>`_ 对内容进行过滤来防止XSS攻击。
     :param int scope, position: 与 `put_text` 函数的同名参数含义一致
     """
     if hasattr(html, '__html__'):
         html = html.__html__()
 
-    spec = _get_output_spec('html', content=html, scope=scope, position=position)
+    spec = _get_output_spec('html', content=html, sanitize=sanitize, scope=scope, position=position)
     return Output(spec)
 
 
-def put_code(content, language='', scope=Scope.Current, position=OutputPosition.BOTTOM) -> Output:
+def put_code(content, language='', rows=None, scope=Scope.Current, position=OutputPosition.BOTTOM) -> Output:
     """
     输出代码块
 
     :param str content: 代码内容
     :param str language: 代码语言
+    :param int rows: 代码块最多可显示的文本行数，默认不限制。内容超出时会使用滚动条。
     :param int scope, position: 与 `put_text` 函数的同名参数含义一致
     """
-    code = "```%s\n%s\n```" % (language, content)
-    return put_markdown(code, scope=scope, position=position)
+    if not isinstance(content, str):
+        content = str(content)
+
+    # For fenced code blocks, escaping the backtick need to use more backticks
+    backticks = '```'
+    while backticks in content:
+        backticks += '`'
+
+    code = "%s%s\n%s\n%s" % (backticks, language, content, backticks)
+    out = put_markdown(code, scope=scope, position=position)
+    if rows is not None:
+        max_height = rows * 19 + 32  # 32 is the code css padding
+        out = style(out, "max-height: %spx" % max_height)
+    return out
 
 
-def put_markdown(mdcontent, strip_indent=0, lstrip=False, scope=Scope.Current,
-                 position=OutputPosition.BOTTOM) -> Output:
+def put_markdown(mdcontent, strip_indent=0, lstrip=False, options=None, sanitize=True,
+                 scope=Scope.Current, position=OutputPosition.BOTTOM) -> Output:
     """
     输出Markdown内容。
 
     :param str mdcontent: Markdown文本
     :param int strip_indent: 对于每一行，若前 ``strip_indent`` 个字符都为空格，则将其去除
     :param bool lstrip: 是否去除每一行开始的空白符
+    :param dict options: 解析Markdown时的配置参数。
+       PyWebIO使用 `marked <https://marked.js.org/>`_ 解析Markdown, 可配置项参见: https://marked.js.org/using_advanced#options (仅支持配置string和boolean类型的项)
+    :param bool sanitize: 是否使用 `DOMPurify <https://github.com/cure53/DOMPurify>`_ 对内容进行过滤来防止XSS攻击。
     :param int scope, position: 与 `put_text` 函数的同名参数含义一致
 
     当在函数中使用Python的三引号语法输出多行内容时，为了排版美观可能会对Markdown文本进行缩进，
@@ -382,7 +404,8 @@ def put_markdown(mdcontent, strip_indent=0, lstrip=False, scope=Scope.Current,
         lines = (i.lstrip() for i in mdcontent.splitlines())
         mdcontent = '\n'.join(lines)
 
-    spec = _get_output_spec('markdown', content=mdcontent, scope=scope, position=position)
+    spec = _get_output_spec('markdown', content=mdcontent, options=options, sanitize=sanitize,
+                            scope=scope, position=position)
     return Output(spec)
 
 
@@ -395,7 +418,7 @@ class span_:
 def span(content, row=1, col=1):
     """用于在 :func:`put_table()` 和 :func:`put_grid()` 中设置内容跨单元格
 
-    :param content: 单元格内容
+    :param content: 单元格内容。可以为字符串或 ``put_xxx()`` 调用。
     :param int row: 竖直方向跨度, 即：跨行的数目
     :param int col: 水平方向跨度, 即：跨列的数目
 
@@ -562,7 +585,7 @@ def put_buttons(buttons, onclick, small=None, link_style=False, scope=Scope.Curr
                 for i in ['primary' , 'secondary' , 'success' , 'danger' , 'warning' , 'info' , 'light' , 'dark']  # ..demo-only
             ], onclick=put_text)  # ..demo-only
 
-    :type onclick: Callable / list
+    :type onclick: callable / list
     :param onclick: 按钮点击回调函数. ``onclick`` 可以是函数或者函数组成的列表.
 
        ``onclick`` 为函数时， 签名为 ``onclick(btn_value)``. ``btn_value`` 为被点击的按钮的 ``value`` 值
@@ -572,7 +595,7 @@ def put_buttons(buttons, onclick, small=None, link_style=False, scope=Scope.Curr
        Tip: 可以使用 ``functools.partial`` 来在 ``onclick`` 中保存更多上下文信息.
 
        Note: 当使用 :ref:`基于协程的会话实现 <coroutine_based_session>` 时，回调函数可以为协程函数.
-    :param bool small: 是否显示小号按钮，默认为False
+    :param bool small: 是否使用小号按钮，默认为False
     :param bool link_style: 是否将按钮显示为链接样式，默认为False
     :param int scope, position: 与 `put_text` 函数的同名参数含义一致
     :param callback_options: 回调函数的其他参数。根据选用的 session 实现有不同参数
@@ -582,8 +605,7 @@ def put_buttons(buttons, onclick, small=None, link_style=False, scope=Scope.Curr
 
        ThreadBasedSession 实现
            * serial_mode: 串行模式模式。默认为 ``False`` ，此时每次触发回调，回调函数会在新线程中立即执行。
-           对于开启了serial_mode的回调，都会在会话内的一个固定线程内执行，当会话运行此回调时，其他所有新的点击事件的回调(包括 ``serial_mode=False`` 的回调)都将排队等待当前点击事件运行完成。
-           如果回调函数运行时间很短，可以开启 ``serial_mode`` 来提高性能。
+           开启serial_mode后，该按钮的回调会在会话内的一个固定线程内串行执行，且其他所有新的点击事件的回调(包括 ``serial_mode=False`` 的回调)都将排队等待当前点击事件运行完成。如果回调函数运行时间很短，可以开启 ``serial_mode`` 来提高性能。
 
     使用示例:
 
@@ -635,12 +657,26 @@ def put_image(src, format=None, title='', width=None, height=None,
               scope=Scope.Current, position=OutputPosition.BOTTOM) -> Output:
     """输出图片。
 
-    :param src: 图片内容. 类型可以为字符串类型的URL或者是 bytes-like object 或者为 ``PIL.Image.Image`` 实例
+    :param src: 图片内容. 可以为: 字符串类型的URL, bytes-like object 表示的图片二进制内容, ``PIL.Image.Image`` 实例
     :param str title: 图片描述
     :param str width: 图像的宽度，可以是CSS像素(数字px)或者百分比(数字%)。
     :param str height: 图像的高度，可以是CSS像素(数字px)或者百分比(数字%)。可以只指定 width 和 height 中的一个值，浏览器会根据原始图像进行缩放。
-    :param str format: 图片格式。如 ``png`` , ``jpeg`` , ``gif`` 等, 仅在 `src` 为非URL时有效
+    :param str format: 图片格式，非必须。如 ``png`` , ``jpeg`` , ``gif`` 等, 仅在 `src` 为非URL时有效
     :param int scope, position: 与 `put_text` 函数的同名参数含义一致
+
+    使用示例:
+
+    .. exportable-codeblock::
+        :name: put_image
+        :summary: 使用`put_image()`输出图片
+
+        from pywebio import STATIC_PATH  # ..demo-only
+        img = open(STATIC_PATH + '/image/favicon_open_32.png', 'rb').read()  # ..demo-only
+        img = open('/path/to/some/image.png', 'rb').read()  # ..doc-only
+        put_image(img, width='50px')
+
+        ## ----
+        put_image('https://www.python.org/static/img/python-logo.png')
     """
     if isinstance(src, PILImage):
         format = src.format
@@ -651,13 +687,15 @@ def put_image(src, format=None, title='', width=None, height=None,
     if isinstance(src, (bytes, bytearray)):
         b64content = b64encode(src).decode('ascii')
         format = '' if format is None else ('image/%s' % format)
+        format = html.escape(format, quote=True)
         src = "data:{format};base64, {b64content}".format(format=format, b64content=b64content)
 
-    width = 'width="%s"' % width if width is not None else ''
-    height = 'height="%s"' % height if height is not None else ''
+    width = 'width="%s"' % html.escape(width, quote=True) if width is not None else ''
+    height = 'height="%s"' % html.escape(height, quote=True) if height is not None else ''
 
-    html = r'<img src="{src}" alt="{title}" {width} {height}/>'.format(src=src, title=title, height=height, width=width)
-    return put_html(html, scope=scope, position=position)
+    tag = r'<img src="{src}" alt="{title}" {width} {height}/>'.format(src=src, title=html.escape(title, quote=True),
+                                                                      height=height, width=width)
+    return put_html(tag, scope=scope, position=position)
 
 
 def put_file(name, content, label=None, scope=Scope.Current, position=OutputPosition.BOTTOM) -> Output:
@@ -674,6 +712,14 @@ def put_file(name, content, label=None, scope=Scope.Current, position=OutputPosi
         在PyWebIO会话(关于会话的概念见 :ref:`Server与script模式 <server_and_script_mode>` )结束后，使用 ``put_file()``
         输出的文件也将无法下载，可以在任务函数末尾处使用 `pywebio.session.hold()` 函数来将会话保持，这样在用户关闭浏览器页面前，
         文件下载将一直可用。
+
+    使用示例:
+
+    .. exportable-codeblock::
+        :name: put_file
+        :summary: `put_file()`的使用
+
+        put_file('hello-world.txt', b'hello world!', 'download me')
     """
     if label is None:
         label = name
@@ -685,11 +731,11 @@ def put_file(name, content, label=None, scope=Scope.Current, position=OutputPosi
 
 def put_link(name, url=None, app=None, new_window=False, scope=Scope.Current,
              position=OutputPosition.BOTTOM) -> Output:
-    """输出链接到其他页面或PyWebIO App的超链接
+    """输出链接到其他网页或PyWebIO App的超链接
 
     :param str name: 链接名称
     :param str url: 链接到的页面地址
-    :param str app: 链接到的PyWebIO应用名
+    :param str app: 链接到的PyWebIO应用名。参见 :ref:`Server模式 <server_and_script_mode>`
     :param bool new_window: 是否在新窗口打开链接
     :param int scope, position: 与 `put_text` 函数的同名参数含义一致
 
@@ -699,8 +745,9 @@ def put_link(name, url=None, app=None, new_window=False, scope=Scope.Current,
 
     href = 'javascript:WebIO.openApp(%r, %d)' % (app, new_window) if app is not None else url
     target = '_blank' if (new_window and url) else '_self'
-    html = '<a href="{href}" target="{target}">{name}</a>'.format(href=href, target=target, name=name)
-    return put_html(html, scope=scope, position=position)
+    tag = '<a href="{href}" target="{target}">{name}</a>'.format(
+        href=html.escape(href, quote=True), target=target, name=html.escape(name))
+    return put_html(tag, scope=scope, position=position)
 
 
 def put_processbar(name, init=0, label=None, auto_close=False, scope=Scope.Current,
@@ -712,6 +759,19 @@ def put_processbar(name, init=0, label=None, auto_close=False, scope=Scope.Curre
     :param str label: 进度条显示的标签. 默认为当前进度的百分比
     :param bool auto_close: 是否在进度完成后关闭进度条
     :param int scope, position: 与 `put_text` 函数的同名参数含义一致
+
+    使用示例:
+
+    .. exportable-codeblock::
+        :name: put_processbar
+        :summary: `put_processbar()` usage
+
+        import time
+
+        put_processbar('bar');
+        for i in range(1, 11):
+            set_processbar('bar', i / 10)
+            time.sleep(0.1)
     """
     processbar_id = 'webio-processbar-%s' % name
     percentage = init * 100
@@ -732,6 +792,8 @@ def set_processbar(name, value, label=None):
     :param str name: 进度条名称
     :param float value: 进度条的值. 范围在 0 ~ 1 之间
     :param str label: 进度条显示的标签. 默认为当前进度的百分比
+
+    参见 `put_processbar()`
     """
     from pywebio.session import run_js
 
@@ -760,15 +822,20 @@ def put_loading(shape='border', color='dark', scope=Scope.Current, position=Outp
      `'warning'` 、`'info'`  、`'light'`  、 `'dark'` (默认)
     :param int scope, position: 与 `put_text` 函数的同名参数含义一致
 
-    .. note::
-        可以通过 :func:`style()` 设置加载提示的尺寸:
+    使用示例:
 
-        .. exportable-codeblock::
-            :name: put_loading-size
-            :summary: `put_loading()`自定义加载提示尺寸
+    .. exportable-codeblock::
+        :name: put_loading
+        :summary: `put_loading()` 使用示例
 
-            put_loading()  # ..demo-only
-            style(put_loading(), 'width:4rem; height:4rem')
+        for shape in ('border', 'grow'):
+            for color in ('primary', 'secondary', 'success', 'danger', 'warning', 'info', 'light', 'dark'):
+                put_text(shape, color)
+                put_loading(shape=shape, color=color)
+
+        ## ----
+        # 可以通过 style() 设置加载提示的尺寸
+        style(put_loading(), 'width:4rem; height:4rem')
     """
     assert shape in ('border', 'grow'), "shape must in ('border', 'grow')"
     assert color in {'primary', 'secondary', 'success', 'danger', 'warning', 'info', 'light', 'dark'}
@@ -776,7 +843,7 @@ def put_loading(shape='border', color='dark', scope=Scope.Current, position=Outp
     html = """<div class="spinner-{shape} text-{color}" role="status">
                 <span class="sr-only">Loading...</span>
             </div>""".format(shape=shape, color=color)
-    return put_html(html, scope=scope, position=position)
+    return put_html(html, sanitize=False, scope=scope, position=position)
 
 
 @safely_destruct_output_when_exp('content')
@@ -788,6 +855,24 @@ def put_collapse(title, content, open=False, scope=Scope.Current, position=Outpu
     :param content: 内容可以为字符串或 ``put_xxx`` 类输出函数的返回值，或者由它们组成的列表。
     :param bool open: 是否默认展开折叠内容。默认不展开内容
     :param int scope, position: 与 `put_text` 函数的同名参数含义一致
+
+    使用示例:
+
+    .. exportable-codeblock::
+        :name: put_collapse
+        :summary: `put_collapse()` 使用示例
+
+        put_collapse('Collapse title', [
+            'text',
+            put_markdown('~~Strikethrough~~'),
+            put_table([
+                ['Commodity', 'Price'],
+                ['Apple', '5.5'],
+            ])
+        ], open=True)
+
+        ## ----
+        put_collapse('Large text', 'Awesome PyWebIO! '*30)
     """
     if not isinstance(content, (list, tuple, OutputList)):
         content = [content]
@@ -805,24 +890,44 @@ def put_collapse(title, content, open=False, scope=Scope.Current, position=Outpu
 
 
 @safely_destruct_output_when_exp('content')
-def put_scrollable(content, max_height=400, horizon_scroll=False, border=True, scope=Scope.Current,
-                   position=OutputPosition.BOTTOM) -> Output:
+def put_scrollable(content, height=400, keep_bottom=False, horizon_scroll=False, border=True,
+                   scope=Scope.Current, position=OutputPosition.BOTTOM, **kwargs) -> Output:
     """固定高度内容输出区域，内容超出则显示滚动条
 
     :type content: list/str/put_xxx()
     :param content: 内容可以为字符串或 ``put_xxx`` 类输出函数的返回值，或者由它们组成的列表。
-    :param int max_height: 区域的最大高度（像素），内容超出次高度则使用滚动条
+    :param int/tuple height: 区域的高度（像素），内容超出此高度则使用滚动条。
+       可以传入 ``(min_height, max_height)`` 来表示高度的范围，比如 ``(100, 200)`` 表示区域高度最小100像素、最高200像素。
+    :param bool keep_bottom: 是否在内容发生变化时自动滚动到底部，默认为 ``False``
     :param bool horizon_scroll: 是否显示水平滚动条
     :param bool border: 是否显示边框
     :param int scope, position: 与 `put_text` 函数的同名参数含义一致
+
+    .. versionchanged:: 1.1
+       添加 ``height`` 参数，移除 ``max_height`` 参数；
+       添加 ``auto_scroll_bottom`` 参数
     """
     if not isinstance(content, (list, tuple, OutputList)):
         content = [content]
 
     for item in content:
-        assert isinstance(item, (str, Output)), "put_collapse() content must be list of str/put_xxx()"
+        assert isinstance(item, (str, Output)), "put_scrollable() content must be list of str/put_xxx()"
 
-    tpl = """<div style="max-height: {{max_height}}px;
+    if 'max_height' in kwargs:
+        import warnings
+        warnings.warn("`max_height` parameter is deprecated in `put_scrollable()`, use `height` instead.",
+                      DeprecationWarning, stacklevel=3)
+        height = kwargs['max_height']  # Backward compatible
+
+    try:
+        min_height, max_height = height
+    except Exception:
+        min_height, max_height = height, height
+
+    dom_id = 'pywebio-%s' % random_str(10)
+
+    tpl = """<div id="{{dom_id}}" {{#keep_bottom}}tabindex="0"{{/keep_bottom}}
+        style="min-height: {{min_height}}px; max-height: {{max_height}}px;
             overflow-y: scroll;
             {{#horizon_scroll}}overflow-x: scroll;{{/horizon_scroll}}
             {{#border}} 
@@ -836,8 +941,23 @@ def put_scrollable(content, max_height=400, horizon_scroll=False, border=True, s
             {{& pywebio_output_parse}}
         {{/contents}}
     </div>"""
+    if keep_bottom:
+        tpl += """
+        <script>
+            (function(){
+                let div = document.getElementById(%r), stop=false;
+                $(div).on('focusin', function(e){ stop=true }).on('focusout', function(e){ stop=false });;
+                new MutationObserver(function (mutations, observe) {
+                    if(!stop) $(div).stop().animate({ scrollTop: $(div).prop("scrollHeight")}, 200);
+                }).observe(div, { childList: true, subtree:true });
+            })();
+        </script>
+        """ % dom_id
+
     return put_widget(template=tpl,
-                      data=dict(contents=content, max_height=max_height, horizon_scroll=horizon_scroll, border=border),
+                      data=dict(dom_id=dom_id, contents=content, min_height=min_height,
+                                max_height=max_height, keep_bottom=keep_bottom,
+                                horizon_scroll=horizon_scroll, border=border),
                       scope=scope, position=position)
 
 
@@ -889,11 +1009,11 @@ def put_widget(template, data, scope=Scope.Current, position=OutputPosition.BOTT
 def put_row(content, size=None, scope=Scope.Current, position=OutputPosition.BOTTOM) -> Output:
     """使用行布局输出内容. 内容在水平方向从左往右排列成一行
 
-    :param list content: 子元素列表, 列表项为 ``put_xxx()`` 调用或者 ``None`` , ``None`` 表示空白列间距
+    :param list content: 子元素列表, 列表项为 ``put_xxx()`` 调用或者 ``None`` , ``None`` 表示空白行间距
     :param str size:
        | 用于指示子元素的宽度, 为空格分割的宽度值列表.
        | 宽度值需要和 ``content`` 中子元素一一对应( ``None`` 子元素也要对应宽度值).
-       | size 默认给 ``None`` 元素分配10像素宽度，将剩余元素平均分配宽度.
+       | size 默认给 ``None`` 元素分配10像素宽度，并为剩余元素平均分配宽度.
 
        宽度值可用格式:
 
@@ -950,7 +1070,7 @@ def _row_column_layout(content, flow, size, scope=Scope.Current, position=Output
         {{#contents}}
             {{& pywebio_output_parse}}
         {{/contents}}
-    </div>""".strip() % style
+    </div>""".strip() % html.escape(style, quote=True)
     return put_widget(template=tpl, data=dict(contents=content), scope=scope,
                       position=position)
 
@@ -961,8 +1081,8 @@ def put_grid(content, cell_width='auto', cell_height='auto', cell_widths=None, c
     """使用网格布局输出内容
 
     :param content: 输出内容. ``put_xxx()`` / None 组成的二维数组, None 表示空白. 数组项可以使用 :func:`span()` 函数设置元素在网格的跨度.
-    :param str cell_width: 网格元素的宽度. 宽度值格式参考 `put_row()` 函数的 ``size`` 参数注释.
-    :param str cell_height: 网格元素的高度. 高度值格式参考 `put_row()` 函数的 ``size`` 参数注释.
+    :param str cell_width: 网格元素的宽度. 宽度值格式参考 `put_row()` 函数的 ``size`` 参数.
+    :param str cell_height: 网格元素的高度. 高度值格式参考 `put_row()` 函数的 ``size`` 参数.
     :param str cell_widths: 网格每一列的宽度. 宽度值用空格分隔. 不可以和 `cell_width` 参数同时使用. 宽度值格式参考 `put_row()` 函数的 ``size`` 参数注释.
     :param str cell_heights: 网格每一行的高度. 高度值用空格分隔. 不可以和 `cell_height` 参数同时使用. 高度值格式参考 `put_row()` 函数的 ``size`` 参数注释.
     :param str direction: 排列方向. 为 ``'row'`` 或 ``'column'`` .
@@ -1028,22 +1148,17 @@ def put_grid(content, cell_width='auto', cell_height='auto', cell_widths=None, c
                 {{& pywebio_output_parse}}
             {{/.}}
         {{/contents}}
-    </div>""".strip() % css
+    </div>""".strip() % html.escape(css, quote=True)
     return put_widget(template=tpl, data=dict(contents=content), scope=scope, position=position)
 
 
-column = put_column
-row = put_row
-grid = put_grid
-
-
 @safely_destruct_output_when_exp('contents')
 def output(*contents):
     """返回一个handler，相当于 ``put_xxx()`` 的占位符，可以传入任何接收 ``put_xxx()`` 调用的地方，通过handler可对自身内容进行修改
 
     output用于对 :ref:`组合输出 <combine_output>` 中的 ``put_xxx()`` 子项进行动态修改（见下方代码示例）
 
-    :param contents: 要输出的初始内容. 元素为 ``put_xxx()`` 形式的调用或字符串，字符串会被看成HTML.
+    :param contents: 要输出的初始内容. 元素为 ``put_xxx()`` 形式的调用或字符串.
     :return: OutputHandler 实例, 实例支持的方法如下:
 
     * ``reset(*contents)`` : 重置内容为 ``contents``
@@ -1053,6 +1168,8 @@ def output(*contents):
        | idx>=0 时表示输出内容到原内容的idx索引的元素的前面；
        | idx<0 时表示输出内容到到原内容的idx索引元素之后.
 
+    其中，参数 ``contents`` 的每一项为 ``put_xxx()``  调用或字符串。
+
     :Example:
 
     .. exportable-codeblock::
@@ -1092,6 +1209,8 @@ def output(*contents):
         @safely_destruct_output_when_exp('outputs')
         def append(self, *outputs):
             for o in outputs:
+                if isinstance(o, str):
+                    o = put_text(o)
                 o.spec['scope'] = _parse_scope(self.scope)
                 o.spec['position'] = OutputPosition.BOTTOM
                 o.send()
@@ -1101,6 +1220,8 @@ def output(*contents):
             """idx可为负，"""
             direction = 1 if idx >= 0 else -1
             for acc, o in enumerate(outputs):
+                if isinstance(o, str):
+                    o = put_text(o)
                 o.spec['scope'] = _parse_scope(self.scope)
                 o.spec['position'] = idx + direction * acc
                 o.send()
@@ -1173,8 +1294,7 @@ def style(outputs, css_style) -> Union[Output, OutputList]:
 
 @safely_destruct_output_when_exp('content')
 def popup(title, content=None, size=PopupSize.NORMAL, implicit_close=True, closable=True):
-    """popup(title, content, size=PopupSize.NORMAL, implicit_close=True, closable=True)
-
+    """
     显示弹窗
 
     ⚠️: PyWebIO不允许同时显示多个弹窗，在显示新弹窗前，会自动关闭页面上存在的弹窗。可以使用 `close_popup()` 主动关闭弹窗
@@ -1184,15 +1304,15 @@ def popup(title, content=None, size=PopupSize.NORMAL, implicit_close=True, closa
     :param content: 弹窗内容. 可以为字符串或 ``put_xxx`` 类输出函数的返回值，或者为它们组成的列表。
     :param str size: 弹窗窗口大小，可选值：
 
-         * ``LARGE`` : 大尺寸
-         * ``NORMAL`` : 普通尺寸
-         * ``SMALL`` : 小尺寸
+         * ``'large'`` : 大尺寸
+         * ``'normal'`` : 普通尺寸
+         * ``'small'`` : 小尺寸
 
     :param bool implicit_close: 是否可以通过点击弹窗外的内容或按下 ``Esc`` 键来关闭弹窗
-    :param bool closable: 是否可由用户关闭弹窗. 默认情况下，用户可以通过点击弹窗右上角的关闭按钮来关闭弹窗，
-       设置为 ``False`` 时弹窗仅能通过 :func:`popup_close()` 关闭， ``implicit_close`` 参数被忽略.
+    :param bool closable: 是否可由用户关闭弹窗. 默认情况下，用户可以通过点击弹窗右上角的关闭按钮来关闭弹窗。
+       设置为 ``False`` 时弹窗仅能通过 :func:`popup_close()` 关闭，此时 ``implicit_close`` 参数将被忽略.
 
-    支持直接传入内容、上下文管理器、装饰器三种形式的调用
+    ``popup()`` 支持直接传入内容、上下文管理器、装饰器三种形式的调用
 
     * 直接传入内容:
 
@@ -1219,13 +1339,13 @@ def popup(title, content=None, size=PopupSize.NORMAL, implicit_close=True, closa
         with popup('Popup title') as s:
             put_html('<h3>Popup Content</h3>')
             put_text('html: <br/>')
-            put_buttons(['clear()'], onclick=lambda _: clear(scope=s))
+            put_buttons([('clear()', s)], onclick=clear)
 
         put_text('Also work!', scope=s)
 
 
-    上下文管理器会开启一个新的输出域并返回Scope名，上下文管理器中的输出调用会显示到弹窗上。
-    上下文管理器退出后，弹窗并不会关闭，依然可以使用 ``scope`` 参数输出内容到弹窗。
+    上下文管理器会开启一个新的输出域并返回Scope名，在上下文管理器中的输出默认会显示到弹窗上。
+    上下文管理器退出后，弹窗并不会关闭，依然可以在输出函数中指定 ``scope`` 参数来输出内容到弹窗。
 
     * 作为装饰器使用:
 
@@ -1261,7 +1381,10 @@ def popup(title, content=None, size=PopupSize.NORMAL, implicit_close=True, closa
 
 
 def close_popup():
-    """关闭当前页面上正在显示的弹窗"""
+    """关闭当前页面上正在显示的弹窗
+
+    参见: `popup()`
+    """
     send_msg(cmd='close_popup')
 
 
@@ -1283,7 +1406,7 @@ def toast(content, duration=2, position='center', color='info', onclick=None):
         :summary: 使用`toast()`显示通知
 
         def show_msg():
-            put_text("Some messages...")
+            put_text("You clicked the notification.")
 
         toast('New messages', position='right', color='#2188ff', duration=0, onclick=show_msg)
 
@@ -1310,10 +1433,10 @@ def use_scope(name=None, clear=False, create_scope=True, **scope_params):
 
     参见 :ref:`用户手册-use_scope() <use_scope>`
 
-    :param name: scope名. 若为None则生成一个全局唯一的scope名.（以上下文管理器形式的调用时，上下文管理器会返回scope名）
+    :param str name: scope名. 若为None则生成一个全局唯一的scope名.（以上下文管理器形式的调用时，上下文管理器会返回scope名）
     :param bool clear: 在进入scope前是否要清除scope里的内容
     :param bool create_scope: scope不存在时是否创建scope
-    :param scope_params: 创建scope时传入set_scope()的参数. 仅在 `create_scope=True` 时有效.
+    :param scope_params: 创建scope时传入 `set_scope()` 的额外的参数. 仅在 `create_scope=True` 时有效.
 
     :Usage:
 
