@@ -226,7 +226,10 @@ class CoroutineBasedSession(Session):
 
 
 class TaskHandle:
-    """协程任务句柄"""
+    """协程任务句柄
+
+    参见：`run_async() <pywebio.session.run_async>`
+    """
 
     def __init__(self, close, closed):
         self._close = close
